bl_info = {
    "name": "Quick Files",
    "author": "Amandeep",
    "description": "Faster way to find and open your blend files",
    "blender": (3, 0, 0),
    "version": (1, 0, 0),
    "warning": "",
    "category": "Object",
}
#region INFORMATION
# Copyright (C) 2023 Amandeep

# Created by Amandeep

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
import bpy
import os
# from .addon_update_checker import *
import time
import shutil
from bpy.app.handlers import persistent
import rna_keymap_ui
preview_collections = {}
preview_list = {}
def preferences():
    return bpy.context.preferences.addons[__package__].preferences
def savePreferences():
    if not os.path.isdir(os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))), "config")):
        os.mkdir(os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(
            os.path.dirname(os.path.abspath(__file__))))), "config"))
    with open(os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))), "config", "RF-Config.txt"), mode='w+', newline='\n', encoding='utf-8') as file:
        for p in preferences().__annotations__.keys():
            
            file.write(f"{p}=>{type(getattr(preferences(),p,'str')).__name__}==={getattr(preferences(),p)}\n")


def loadPreferences(dummy1=None, dummy2=None):
    if not os.path.isdir(os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))), "config")):
        os.mkdir(os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(
            os.path.dirname(os.path.abspath(__file__))))), "config"))
    if os.path.isfile(os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))), "config", "RF-Config.txt")):
        with open(os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))), "config", "RF-Config.txt"), mode='r', newline='\n', encoding='utf-8') as file:
            prefs = file.readlines()
            for p in reversed(prefs):
                try:
                    
                    attr = p[:p.index("=>")]
                    type = p[p.index("=>")+2:p.index("===")]
                    if type=='String':
                        type='str'
                    value = p[p.index("===")+3:]
                    value = str(value.replace("\n", ""))
                    if type=='bool':
                        setattr(preferences(), attr,eval(value))
                    else:
                        setattr(preferences(), attr,value)
                except Exception as e:
                    pass
@persistent
def refresh_files(self=None,context=None):
    bpy.ops.rf.refreshfiles('INVOKE_DEFAULT')


def rename_file(file_path, new_name):
    directory = os.path.dirname(file_path)
    base, extension = os.path.splitext(file_path)
    new_name = new_name + extension
    new_path = os.path.join(directory, new_name)
    os.rename(file_path, new_path)

def delete_file(file_path):
    os.remove(file_path)

def duplicate_file(file_path, new_name=None):
    directory = os.path.dirname(file_path)
    base, extension = os.path.splitext(file_path)
    if not new_name:
        new_name = base + "_copy" + extension
    new_path = os.path.join(directory, new_name)
    i = 1
    while os.path.exists(new_path):
        new_name = base + "_copy" + str(i) + extension
        new_path = os.path.join(directory, new_name)
        i += 1
    shutil.copy2(file_path, new_path)
def find_blend_files(root_dir, sort_by="NAME", reverse=False):
    blend_files = []
    for root, dirs, files in os.walk(root_dir):
        for file in files:
            if file.endswith(".blend"):
                file_path = os.path.join(root, file)
                mtime = os.path.getmtime(file_path)
                time_str = time.strftime("%d %b %Y %H:%M", time.localtime(mtime))
                blend_files.append((file, file_path, time_str,mtime))
        if not preferences().include_subfolders:
            break
    if sort_by == "NAME":
        blend_files.sort(key=lambda x: x[0], reverse=reverse)
    elif sort_by == "TIME":
        blend_files.sort(key=lambda x: x[3], reverse=reverse)
    return blend_files
def draw_hotkeys(col,km_name):
    kc = bpy.context.window_manager.keyconfigs.user
    for kmi in [a.idname for b,a in addon_keymaps]:
        km2=kc.keymaps[km_name]
        kmi2=[]
        for a,b in km2.keymap_items.items():
            if a==kmi:
                kmi2.append(b)
        if kmi2:
            for a in kmi2:
                col.context_pointer_set("keymap", km2)
                rna_keymap_ui.draw_kmi([], kc, km2, a, col, 0)
class RFPrefs(bpy.types.AddonPreferences):
    bl_idname = __package__
    blends_path:bpy.props.StringProperty(default="Choose Path",name="Blend's Directory",subtype='DIR_PATH',update=refresh_files)
    include_subfolders:bpy.props.BoolProperty(default=True,name="Include Subfolders",update=refresh_files)
    sort_by:bpy.props.EnumProperty(items=[('NAME','Name','Sort by name'),('TIME','Time','Sort by time')],name="Sort by",default=1)
    reverse:bpy.props.BoolProperty(default=True,name="Reverse")

    def draw(self, context):
        layout=self.layout
        # draw_update_section_for_prefs(layout,context)
        draw_hotkeys(layout, "3D View")
        # layout.prop(self,'blends_path')
class RF_OT_Rename(bpy.types.Operator):
    bl_idname = 'rf.renamefile'
    bl_label = 'Rename'
    bl_description = ""
    bl_options = { 'UNDO',}
    file_path:bpy.props.StringProperty()
    new_name:bpy.props.StringProperty(name='New Name')
    def draw(self,context):
        self.layout.prop(self,'new_name')
    def execute(self, context):
        rename_file(file_path=self.file_path,new_name=self.new_name)
        bpy.ops.rf.refreshfiles('INVOKE_DEFAULT')
        return {'FINISHED'}
    def invoke(self,context,event):
        return context.window_manager.invoke_props_dialog(self)
class RF_OT_Delete(bpy.types.Operator):
    bl_idname = 'rf.delete'
    bl_label = 'Delete'
    bl_description = ""
    bl_options = { 'UNDO',}
    file_path:bpy.props.StringProperty()
    def execute(self, context):
        delete_file(self.file_path)
        bpy.ops.rf.refreshfiles('INVOKE_DEFAULT')
        return {'FINISHED'}
    def invoke(self,context,event):
        return context.window_manager.invoke_confirm(self,event)
class RF_OT_Duplicate(bpy.types.Operator):
    bl_idname = 'rf.duplicate'
    bl_label = 'Duplicate'
    bl_description = ""
    bl_options = { 'UNDO',}
    file_path:bpy.props.StringProperty()
    def execute(self, context):
        duplicate_file(self.file_path)
        bpy.ops.rf.refreshfiles('INVOKE_DEFAULT')
        return {'FINISHED'}
    def invoke(self,context,event):

        return self.execute(context)
class RF_OT_Append(bpy.types.Operator):
    bl_idname = 'rf.append'
    bl_label = 'Append Data'
    bl_description = "Append Data from this file\nCTRL+LMB:Link Data from this File"
    bl_options = {'UNDO',}
    file_path:bpy.props.StringProperty()

    def invoke(self,context,event):
        if event.ctrl:
            bpy.ops.wm.link('INVOKE_DEFAULT',filepath=self.file_path)
        else:
            bpy.ops.wm.append('INVOKE_DEFAULT',filepath=self.file_path)
        return {'FINISHED'}
class RF_OT_Change_Sort_Settings(bpy.types.Operator):
    bl_idname = 'rf.changesort'
    bl_label = ''
    bl_description = ""
    bl_options = { 'UNDO',}
    sort_by:bpy.props.StringProperty()
    
    def execute(self, context):
        if self.sort_by=='NAME':
            if preferences().sort_by=='NAME':
                preferences().reverse=not preferences().reverse
            else:
                preferences().sort_by='NAME'
                preferences().reverse=False
        else:
            if preferences().sort_by=='TIME':
                preferences().reverse=not preferences().reverse
            else:
                preferences().sort_by='TIME'
                preferences().reverse=True
        bpy.ops.rf.refreshfiles('INVOKE_DEFAULT')
        return {'FINISHED'}
class RF_OT_Load_Files(bpy.types.Operator):
    bl_idname = 'rf.refreshfiles'
    bl_label = 'Refresh'
    bl_description = "Refresh"
    bl_options = { 'UNDO',}
    def execute(self, context):
        context.scene.blend_files.clear()
        files=find_blend_files(preferences().blends_path,sort_by=preferences().sort_by,reverse=preferences().reverse)
        # t=context.scene.blend_files.add()
        # t.name='sort_settings'
        for f in files:
            if context.scene.file_search.lower() in f[0].lower():
                t=context.scene.blend_files.add()

                t.name=f[0]
                t.path=f[1]
                t.time=f[2]
        return {'FINISHED'}
class RF_OT_Call_Panel(bpy.types.Operator):
    bl_idname = 'rf.callpanel'
    bl_label = 'Open Quick Files Popup'
    bl_description = "Open Quick Files Popup"
    bl_options = { 'UNDO',}
    def draw(self, context):
        layout= self.layout
        layout.label(text='',icon='GRIP')
        layout.prop(preferences(),'blends_path',text="")
        row=layout.row()
        row.operator("rf.refreshfiles",text='Reload',icon='FILE_REFRESH')
        row.prop(preferences(),'include_subfolders',toggle=True,icon='CHECKBOX_DEHLT' if not preferences().include_subfolders else 'CHECKBOX_HLT',emboss=False)
        box=layout.box()
        box.prop(context.scene,'file_search',icon='VIEWZOOM',text='')
        
        row = box.row(align=True)
        row=row.split(factor=0.48,align=True)

        row2=row.row(align=True)
        row2=row2.split(factor=0.9)
        row2.operator("rf.changesort",text="Name",icon=("TRIA_UP" if preferences().reverse else 'TRIA_DOWN') if preferences().sort_by=='NAME' else 'BLANK1',emboss=False).sort_by='NAME'
        row2.label(text=' ',icon='BLANK1')
        row2=row.row(align=True)
        row2=row2.split(factor=0.9)
        row2.operator("rf.changesort",text="Date Modified",icon=("TRIA_UP" if not preferences().reverse else 'TRIA_DOWN') if preferences().sort_by=='TIME' else 'BLANK1',emboss=False).sort_by='TIME'
        row2.label(text=' ',icon='BLANK1')
        box.template_list("RF_UL_Files_List", "", context.scene, "blend_files",
                                context.scene, "selected_blend_file_index" ,item_dyntip_propname='name',rows=10)
        if context.scene.blend_files and context.scene.selected_blend_file_index>-1 and context.scene.selected_blend_file_index<=len(context.scene.blend_files):
            row=box.row()
            op=row.operator("wm.open_mainfile",icon='FILE_BLEND')
            op.filepath=context.scene.blend_files[context.scene.selected_blend_file_index].path
            op.display_file_selector=False
            row.operator("rf.append",icon='APPEND_BLEND').file_path=context.scene.blend_files[context.scene.selected_blend_file_index].path
            row=box.row()
            row.operator("rf.renamefile",icon='FILE_TEXT').file_path=context.scene.blend_files[context.scene.selected_blend_file_index].path
            row.operator("rf.delete",icon='TRASH').file_path=context.scene.blend_files[context.scene.selected_blend_file_index].path
            row.operator("rf.duplicate",icon='DUPLICATE').file_path=context.scene.blend_files[context.scene.selected_blend_file_index].path
    def execute(self, context):
        return {'FINISHED'}
    def invoke(self,context,event):
        return context.window_manager.invoke_popup(self,width=400)
class RF_FilesInfo(bpy.types.PropertyGroup):
    name: bpy.props.StringProperty(name="name", default="")
    path:bpy.props.StringProperty()
    time:bpy.props.StringProperty()
class RF_PT_Recent_Files(bpy.types.Panel):
    bl_label = "Recent Files"
    bl_idname = "RF_PT_Recent_Files"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Files"
    
    def draw(self, context):
        layout= self.layout
        
        #layout.operator("rtools.importasdecal")
        layout.template_recent_files(rows=10)
class RF_UL_Files_List(bpy.types.UIList):
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname):
        ob = data

        obj = item
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            if obj:
                row = layout.row()
                row.label(text=obj.name)
                row.label(text=obj.time)
class RF_PT_Open_Files(bpy.types.Panel):
    bl_label = "Open Files"
    bl_idname = "RF_PT_Open_Files"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Files"
    
    def draw(self, context):
        layout= self.layout
        # draw_update_section_for_panel(layout,context)
        layout.prop(preferences(),'blends_path',text="")
        row=layout.row()
        row.operator("rf.refreshfiles",text='Reload',icon='FILE_REFRESH')
        row.prop(preferences(),'include_subfolders',toggle=True,icon='CHECKBOX_DEHLT' if not preferences().include_subfolders else 'CHECKBOX_HLT',emboss=False)
        box=layout.box()
        box.prop(context.scene,'file_search',icon='VIEWZOOM',text='')
        
        row = box.row(align=True)
        row=row.split(factor=0.48,align=True)

        row2=row.row(align=True)
        row2=row2.split(factor=0.9)
        row2.operator("rf.changesort",text="Name",icon=("TRIA_UP" if preferences().reverse else 'TRIA_DOWN') if preferences().sort_by=='NAME' else 'BLANK1',emboss=False).sort_by='NAME'
        row2.label(text=' ',icon='BLANK1')
        row2=row.row(align=True)
        row2=row2.split(factor=0.9)
        row2.operator("rf.changesort",text="Date Modified",icon=("TRIA_UP" if not preferences().reverse else 'TRIA_DOWN') if preferences().sort_by=='TIME' else 'BLANK1',emboss=False).sort_by='TIME'
        row2.label(text=' ',icon='BLANK1')
        box.template_list("RF_UL_Files_List", "", context.scene, "blend_files",
                                context.scene, "selected_blend_file_index" ,item_dyntip_propname='name',rows=10)
        if context.scene.blend_files and context.scene.selected_blend_file_index>-1 and context.scene.selected_blend_file_index<=len(context.scene.blend_files):
            row=box.row()
            op=row.operator("wm.open_mainfile",icon='FILE_BLEND')
            op.filepath=context.scene.blend_files[context.scene.selected_blend_file_index].path
            op.display_file_selector=False
            row.operator("rf.append",icon='APPEND_BLEND').file_path=context.scene.blend_files[context.scene.selected_blend_file_index].path
            row=box.row()
            row.operator("rf.renamefile",icon='FILE_TEXT').file_path=context.scene.blend_files[context.scene.selected_blend_file_index].path
            row.operator("rf.delete",icon='TRASH').file_path=context.scene.blend_files[context.scene.selected_blend_file_index].path
            row.operator("rf.duplicate",icon='DUPLICATE').file_path=context.scene.blend_files[context.scene.selected_blend_file_index].path
            
classes = (RF_UL_Files_List,RF_OT_Duplicate,RF_OT_Delete,RF_OT_Rename,RF_OT_Append,RF_OT_Call_Panel,
    RF_PT_Open_Files,RF_PT_Recent_Files,RF_FilesInfo,RFPrefs,RF_OT_Load_Files,RF_OT_Change_Sort_Settings
)

icon_collection={}
addon_keymaps = []

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)
    
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    kmaps = [('rf.callpanel','F6',''),
    ]

    km = kc.keymaps.new(name="3D View", space_type="VIEW_3D")
    if kc:
        for (op, k, sp) in kmaps:

            kmi = km.keymap_items.new(
                op,
                type=k,
                value="PRESS",
                alt="alt" in sp,
                shift="shift" in sp,
                ctrl="ctrl" in sp,
            )
            addon_keymaps.append((km, kmi))
    bpy.types.Scene.blend_files = bpy.props.CollectionProperty(type=RF_FilesInfo)
    bpy.types.Scene.selected_blend_file_index=bpy.props.IntProperty(default=0)
    bpy.types.Scene.file_search=bpy.props.StringProperty(name='Search',update=refresh_files)
    bpy.app.handlers.load_post.append(loadPreferences)
    bpy.app.handlers.load_post.append(refresh_files)
    try:
        loadPreferences()
    except:
        pass
    # addon_update_checker.register("538a22769e0c0203d3a28e1ea5810b72")
    # bpy.types.Scene.rf_directories= bpy.props.EnumProperty(items=rf_directories,name="Folder")
def unregister():
    savePreferences()
    from bpy.utils import unregister_class

    for cls in reversed(classes):
        unregister_class(cls)
    for (km, kmi) in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()
    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)
    preview_collections.clear()
    preview_list.clear()
    try:
        bpy.app.handlers.load_post.remove(refresh_files)
    except:
        pass
    try:
        bpy.app.handlers.load_post.remove(loadPreferences)
    except:
        pass
    # addon_update_checker.unregister()
if __name__ == "__main__":
    register()

